@@ -49,6 +49,7 @@ class RedisConnection(object):
 
         cls.config = config.copy()
         modified_config = config.copy()
+        modified_config.pop('shard_manager_enabled', False)
         if modified_config.pop('enabled', True) is False:
             raise NotImplementedError("The Redis connection has been disabled.")
         address = modified_config.pop('host'), modified_config.pop('port')
@@ -64,6 +65,15 @@ class RedisConnection(object):
         conn = cls.pool
         return cls(conn)
 
+    @classmethod
+    def _get_connection(cls):
+        """
+        Acquires a connection from the connection pool.
+        """
+
+        conn = cls.pool
+        return cls(conn)
+
     async def disconnect(self) -> None:
         """
         Releases a connection back into the connection pool.
