@@ -5,6 +5,9 @@ import sys
 import typing
 import os
 import importlib
+from voxelbotutils.cogs.utils.shard_manager import ShardManager
+
+import toml
 
 from .cogs.utils.database import DatabaseConnection
 from .cogs.utils.redis import RedisConnection
@@ -149,7 +152,7 @@ def _set_default_log_level(logger_name, log_filter, formatter, loglevel):
     # logger.critical("Test critical message")
 
 
-def set_default_log_levels(bot: Bot, args: argparse.Namespace) -> None:
+def set_default_log_levels(args: argparse.Namespace) -> None:
     """
     Set the default levels for the logger
 
@@ -161,21 +164,21 @@ def set_default_log_levels(bot: Bot, args: argparse.Namespace) -> None:
     # formatter = logging.Formatter('%(asctime)s [%(levelname)s][%(name)s] %(message)s')
     # formatter = logging.Formatter('{asctime} | {levelname: <8} | {module}:{funcName}:{lineno} - {message}', style='{')
     formatter = logging.Formatter('{asctime} | {levelname: <8} | {name}: {message}', style='{')
-    bot.logger = logger
-
     log_filter = LogFilter(logging.WARNING)
-
     loggers = [
-        bot.logger,
-        bot.database.logger,
-        bot.redis.logger,
-        bot.stats.logger,
+        logger,
+        DatabaseConnection.logger,
+        RedisConnection.logger,
+        StatsdConnection.logger,
         'discord',
         'aiohttp',
         'aiohttp.access',
         'upgradechat',
+        # 'shardmanager',
     ]
     for i in loggers:
+        if i is None:
+            continue
         _set_default_log_level(i, log_filter, formatter, args.loglevel)
 
 
@@ -235,7 +238,7 @@ async def start_database_pool(config: dict) -> None:
         await create_initial_database(db)
 
 
-async def start_redis_pool(config:dict) -> None:
+async def start_redis_pool(config: dict) -> None:
     """
     Start the redis pool conneciton
     """
@@ -256,17 +259,7 @@ async def start_redis_pool(config:dict) -> None:
     logger.info("Created redis pool successfully")
 
 
-def run_bot(args: argparse.Namespace) -> None:
-    """
-    Starts the bot, connects the database, runs the async loop forever
-
-    Args:
-        args (argparse.Namespace): The arguments namespace that wants to be run
-    """
-
-    os.chdir(args.bot_directory)
-
-    # Use right event loop
+def set_event_loop():
     try:
         import uvloop
         asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
@@ -276,11 +269,29 @@ def run_bot(args: argparse.Namespace) -> None:
         loop = asyncio.ProactorEventLoop()
         asyncio.set_event_loop(loop)
 
+
+def run_bot(args: argparse.Namespace) -> None:
+    """
+    Starts the bot, connects the database, runs the async loop forever
+
+    Args:
+        args (argparse.Namespace): The arguments namespace that wants to be run
+    """
+
+    os.chdir(args.bot_directory)
+    set_event_loop()
+
     # And run file
     shard_ids = validate_sharding_information(args)
     bot = Bot(shard_count=args.shardcount, shard_ids=shard_ids, config_file=args.config_file)
     loop = bot.loop
-    set_default_log_levels(bot, args)
+
+    # Set up loggers
+    bot.logger = logger
+    DatabaseConnection.logger = logger.getChild("database")
+    RedisConnection.logger = logger.getChild("redis")
+    StatsdConnection.logger = logger.getChild("statsd")
+    set_default_log_levels(args)
 
     # Connect the database pool
     if bot.config.get('database', {}).get('enabled', False):
@@ -341,16 +352,7 @@ def run_website(args: argparse.Namespace) -> None:
     import markdown
 
     os.chdir(args.website_directory)
-
-    # Use right event loop
-    try:
-        import uvloop
-        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
-    except ImportError:
-        pass
-    if sys.platform == 'win32':
-        loop = asyncio.ProactorEventLoop()
-        asyncio.set_event_loop(loop)
+    set_event_loop()
 
     # Read config
     with open(args.config_file) as a:
@@ -425,12 +427,12 @@ def run_website(args: argparse.Namespace) -> None:
 
     # Add our connections and their loggers
     app['database'] = DatabaseConnection
-    DatabaseConnection.logger = logger.getChild("database")
     app['redis'] = RedisConnection
-    RedisConnection.logger = logger.getChild("redis")
     app['logger'] = logger.getChild("route")
-    StatsdConnection.logger = logger.getChild("statsd")
     app['stats'] = StatsdConnection
+    DatabaseConnection.logger = logger.getChild("database")
+    RedisConnection.logger = logger.getChild("redis")
+    StatsdConnection.logger = logger.getChild("statsd")
 
     # Add our config
     app['config'] = config
@@ -453,7 +455,7 @@ def run_website(args: argparse.Namespace) -> None:
         bot = Bot(f"./config/{bot_config_location}")
         app['bots'][bot_name] = bot
         if index == 0:
-            set_default_log_levels(bot, args)
+            set_default_log_levels(args)
         try:
             loop.run_until_complete(bot.login())
             bot.load_all_extensions()
@@ -494,3 +496,48 @@ def run_website(args: argparse.Namespace) -> None:
     logger.info("Closing asyncio loop")
     loop.stop()
     loop.close()
+
+
+def run_sharder(args: argparse.Namespace) -> None:
+    """
+    Starts the sharder, connects the redis, runs the async loop forever
+
+    Args:
+        args (argparse.Namespace): The arguments namespace that wants to be run
+    """
+
+    set_event_loop()
+    loop = asyncio.get_event_loop()
+
+    # Read config
+    with open(args.config_file) as a:
+        config = toml.load(a)
+
+    RedisConnection.logger = logger.getChild("redis")
+    set_default_log_levels(args)
+
+    # Connect the redis pool
+    if config.get('redis', {}).get('enabled', False):
+        re_connect = start_redis_pool(config)
+        loop.run_until_complete(re_connect)
+    else:
+        raise Exception("Redis needs to be enabled to be able to run the sharder.")
+
+    # Get the max concurrency
+    max_concurrency = loop.run_until_complete(ShardManager.get_max_concurrency(config['token']))
+
+    # Run the bot
+    logger.info(f"Running sharder with {max_concurrency} shards")
+    loop.create_task(ShardManager(max_concurrency).run())
+    try:
+        loop.run_forever()
+    except KeyboardInterrupt:
+        logger.info("Logging out sharder")
+
+    # We're now done running the sharder, time to clean up and close
+    logger.info("Closing redis pool")
+    RedisConnection.pool.close()
+
+    logger.info("Closing asyncio loop")
+    loop.stop()
+    loop.close()
